/*[Body parameters]*/
slide_center_distance=66; //0.1
//Slide external diameter including clearence if needed
slide_dia=8; //0.01
slide_tapping_dia=7.2; //0.01
//Springs concentric with slides
spring_cuttout_dia=12; //0.05
//Needs to be less than stamp bolt thread lenght with plenty to spare
spring_cuttout_upper_depth=15; //0.05
spring_cuttout_lower_depth=5; //0.05
wall_thickness=2; //0.1

/*[Bolt parameters]*/
//External diameter including clearence if needed
thread_tapping_dia=5.2; //0.01
//Length needs to be fine tuned so bolt end is flush with die faces
thread_length=20.2; //0.01
//Unthreaded lenght if needed
waist_dia=6.2; //0.01
waist_length=2; //0.01
//bolt head diameter plus clearence
head_dia=10; //0.01
head_height=6.1; //0.01

/*[Washer parameters]*/
washer_thickness=2.4; //0.05
washer_id=13.2; //0.01
washer_od=23.8; //0.01

/*[Guide plate parameters]*/
guide_length=42; //0.05
guide_width=16; //0.05
guide_thickness=0.7; //0.01
lip_width=1; //0.05
lip_depth=2; //0.05
lip_clearence=0.2; //0.01

/*[Stamping parameters]*/
stamp_depth=1.2; //0.01
centre_distance=28; //0.1
//Extra pressure between washers to allow for plastic compression
washer_preload=0.1; //0.05

/*[Show parts]*/
show_lower=true;
show_upper=true;
show_guide=true;

//empty module used to hide values below in customizer
module customizer_separator() {}
/////////////////////////

if(show_lower==true) lower();
if(show_upper==true) translate([washer_od+(wall_thickness*3),0,0]) upper();
if(show_guide==true) translate([(washer_od*2)+(wall_thickness*6)+lip_width,0,0]) guide();

module upper(){
	total_height=	thread_length+
								waist_length+
								head_height-
								stamp_depth;

	difference(){
		//body exterior
		linear_extrude(height=total_height){
			hull(){
				translate([0,slide_center_distance/2])
					circle((spring_cuttout_dia/2)+wall_thickness,$fn=20);
				translate([0,-slide_center_distance/2])
					circle((spring_cuttout_dia/2)+wall_thickness,$fn=20);
				translate([0,centre_distance/2]) 
					circle((washer_od/2)+wall_thickness,$fn=30);
				translate([0,-centre_distance/2])
					circle((washer_od/2)+wall_thickness,$fn=30);
			}
		}
		//bolt/slide hole cuttouts
		translate([0,0,-0.1]) linear_extrude(height=total_height+0.2){
			translate([0,slide_center_distance/2])
				circle(slide_dia/2,$fn=20);
			translate([0,-slide_center_distance/2])
				circle(slide_dia/2,$fn=20);
			translate([0,centre_distance/2]) 
				circle(thread_tapping_dia/2,$fn=20);
			translate([0,-centre_distance/2])
				circle(thread_tapping_dia/2,$fn=20);
		}
		//bolt waisted length cuttouts
		translate([0,0,thread_length])
			linear_extrude(height=waist_length+0.1){
				translate([0,centre_distance/2]) 
					circle(waist_dia/2,$fn=20);
				translate([0,-centre_distance/2])
					circle(waist_dia/2,$fn=20);
		}
		//bolt head cuttouts
		translate([0,0,thread_length+waist_length])
			linear_extrude(height=head_height-stamp_depth+0.1){
				translate([0,centre_distance/2]) 
					circle(head_dia/2,$fn=20);
				translate([0,-centre_distance/2])
					circle(head_dia/2,$fn=20);
		}
		//die spring cuttouts
		translate([0,0,total_height-spring_cuttout_upper_depth])
			linear_extrude(height=spring_cuttout_upper_depth+0.1){
				translate([0,slide_center_distance/2]) 
					circle(spring_cuttout_dia/2,$fn=20);
				translate([0,-slide_center_distance/2])
					circle(spring_cuttout_dia/2,$fn=20);
		}
		//washer cuttouts
		translate([0,0,total_height-washer_thickness+(washer_preload/2)])
			linear_extrude(height=washer_thickness+(washer_preload/2)+0.1){
				translate([0,centre_distance/2])
					difference(){
						circle(washer_od/2,$fn=30);
						circle(washer_id/2,$fn=30);
				}
				translate([0,-centre_distance/2])
					difference(){
						circle(washer_od/2,$fn=30);
						circle(washer_id/2,$fn=30);
				}
		}
		//washer support reenforcement slits
		translate([0,0,-0.1])
			linear_extrude(height=total_height+0.2){
				translate([0,centre_distance/2])
					difference(){
						circle((washer_id/2)+0.1,$fn=30);
						circle(washer_id/2,$fn=30);
				}
				translate([0,-centre_distance/2])
					difference(){
						circle((washer_id/2)+0.1,$fn=30);
						circle(washer_id/2,$fn=30);
				}
		}
	}
}

module lower(){
	total_height=	thread_length+
								waist_length+
								head_height+
								stamp_depth;

	difference(){
		//body exterior
		linear_extrude(height=total_height){
			hull(){
				translate([0,slide_center_distance/2])
					circle((spring_cuttout_dia/2)+wall_thickness,$fn=20);
				translate([0,-slide_center_distance/2])
					circle((spring_cuttout_dia/2)+wall_thickness,$fn=20);
				translate([0,centre_distance/2]) 
					circle((washer_od/2)+wall_thickness,$fn=30);
				translate([0,-centre_distance/2])
					circle((washer_od/2)+wall_thickness,$fn=30);
			}
		}
		//bolt/slide hole cuttouts
		translate([0,0,-0.1]) linear_extrude(height=total_height+0.2){
			translate([0,slide_center_distance/2])
				circle(slide_tapping_dia/2,$fn=20);
			translate([0,-slide_center_distance/2])
				circle(slide_tapping_dia/2,$fn=20);
			translate([0,centre_distance/2]) 
				circle(thread_tapping_dia/2,$fn=20);
			translate([0,-centre_distance/2])
				circle(thread_tapping_dia/2,$fn=20);
		}
		//bolt waisted length cuttouts
		translate([0,0,thread_length])
			linear_extrude(height=waist_length+0.1){
				translate([0,centre_distance/2]) 
					circle(waist_dia/2,$fn=20);
				translate([0,-centre_distance/2])
					circle(waist_dia/2,$fn=20);
		}
		//bolt head cuttouts
		translate([0,0,thread_length+waist_length])
			linear_extrude(height=head_height+stamp_depth+0.1){
				translate([0,centre_distance/2]) 
					circle(head_dia/2,$fn=20);
				translate([0,-centre_distance/2])
					circle(head_dia/2,$fn=20);
		}
		//die spring cuttouts
		translate([0,0,total_height-spring_cuttout_lower_depth])
			linear_extrude(height=spring_cuttout_lower_depth+0.1){
				translate([0,slide_center_distance/2]) 
					circle(spring_cuttout_dia/2,$fn=20);
				translate([0,-slide_center_distance/2])
					circle(spring_cuttout_dia/2,$fn=20);
		}
		//washer cuttouts
		translate([0,0,total_height-washer_thickness+(washer_preload/2)])
			linear_extrude(height=washer_thickness+(washer_preload/2)+0.1){
				translate([0,centre_distance/2])
					difference(){
						circle(washer_od/2,$fn=30);
						circle(washer_id/2,$fn=30);
				}
				translate([0,-centre_distance/2])
					difference(){
						circle(washer_od/2,$fn=30);
						circle(washer_id/2,$fn=30);
				}
		}
		//washer support reenforcement slits
		translate([0,0,-0.1])
			linear_extrude(height=total_height+0.2){
				translate([0,centre_distance/2])
					difference(){
						circle((washer_id/2)+0.1,$fn=30);
						circle(washer_id/2,$fn=30);
				}
				translate([0,-centre_distance/2])
					difference(){
						circle((washer_id/2)+0.1,$fn=30);
						circle(washer_id/2,$fn=30);
				}
		}
	}
}

module guide(){
	guide_overhang=	wall_thickness+
									lip_width+
									lip_clearence;

	//flat area
	translate([0,0,0]) linear_extrude(height=guide_thickness){
		difference(){
			hull(){
				translate([0,slide_center_distance/2])
					circle((spring_cuttout_dia/2)+guide_overhang,$fn=20);
				translate([0,-slide_center_distance/2])
					circle((spring_cuttout_dia/2)+guide_overhang,$fn=20);
				translate([0,centre_distance/2]) 
					circle((washer_od/2)+guide_overhang,$fn=30);
				translate([0,-centre_distance/2])
					circle((washer_od/2)+guide_overhang,$fn=30);
			}
			//workpiece cuttout
			square([guide_width,guide_length],true);
			//spring cuttouts
			translate([0,slide_center_distance/2]) 
				circle(spring_cuttout_dia/2,$fn=20);
			translate([0,-slide_center_distance/2]) 
				circle(spring_cuttout_dia/2,$fn=20);
		}
	}
	
	translate([0,0,guide_thickness-0.1]) linear_extrude(height=lip_depth+0.1){
		difference(){
			hull(){
				translate([0,slide_center_distance/2])
					circle((spring_cuttout_dia/2)+guide_overhang,$fn=20);
				translate([0,-slide_center_distance/2])
					circle((spring_cuttout_dia/2)+guide_overhang,$fn=20);
				translate([0,centre_distance/2]) 
					circle((washer_od/2)+guide_overhang,$fn=30);
				translate([0,-centre_distance/2])
					circle((washer_od/2)+guide_overhang,$fn=30);
			}
			hull(){
				translate([0,slide_center_distance/2])
					circle((spring_cuttout_dia/2)+wall_thickness+lip_clearence,$fn=20);
				translate([0,-slide_center_distance/2])
					circle((spring_cuttout_dia/2)+wall_thickness+lip_clearence,$fn=20);
				translate([0,centre_distance/2]) 
					circle((washer_od/2)+wall_thickness+lip_clearence,$fn=30);
				translate([0,-centre_distance/2])
					circle((washer_od/2)+wall_thickness+lip_clearence,$fn=30);
			}
		}
	}
}
